@@ -2,20 +2,19 @@
 /*
  * Copyright (C) 2007 Collabora Ltd.
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2.1 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
- * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  * 
  * Authors: Xavier Claessens <xclaesse@gmail.com>
  */
@@ -100,6 +99,8 @@ static void       status_icon_message_received_cb (EmpathyTpChat          *tp_ch
 static void       status_icon_idle_notify_cb      (EmpathyStatusIcon      *icon);
 static void       status_icon_update_tooltip      (EmpathyStatusIcon      *icon);
 static void       status_icon_set_from_state      (EmpathyStatusIcon      *icon);
+static void       status_icon_set_visibility      (EmpathyStatusIcon      *icon,
+						   gboolean                visible);
 static void       status_icon_toggle_visibility   (EmpathyStatusIcon      *icon);
 static void       status_icon_activate_cb         (GtkStatusIcon          *status_icon,
 						   EmpathyStatusIcon      *icon);
@@ -230,7 +231,6 @@ empathy_status_icon_new (GtkWindow *window)
 	EmpathyStatusIconPriv *priv;
 	EmpathyStatusIcon     *icon;
 	gboolean               should_hide;
-	gboolean               visible;
 
 	g_return_val_if_fail (GTK_IS_WINDOW (window), NULL);
 
@@ -246,10 +246,9 @@ empathy_status_icon_new (GtkWindow *window)
 	empathy_conf_get_bool (empathy_conf_get (),
 			      EMPATHY_PREFS_UI_MAIN_WINDOW_HIDDEN,
 			      &should_hide);
-	visible = empathy_window_get_is_visible (window);
 
-	if ((!should_hide && !visible) || (should_hide && visible)) {
-		status_icon_toggle_visibility (icon);
+	if (gtk_window_is_active (priv->window) == should_hide) {
+		status_icon_set_visibility (icon, !should_hide);
 	}
 
 	return icon;
@@ -391,16 +390,14 @@ status_icon_set_from_state (EmpathyStatusIcon *icon)
 }
 
 static void
-status_icon_toggle_visibility (EmpathyStatusIcon *icon)
+status_icon_set_visibility (EmpathyStatusIcon *icon,
+			    gboolean           visible)
 {
 	EmpathyStatusIconPriv *priv;
-	gboolean               visible;
 
 	priv = GET_PRIV (icon);
 
-	visible = gtk_window_has_toplevel_focus (GTK_WINDOW (priv->window));
-
-	if (visible) {
+	if (!visible) {
 		empathy_window_iconify (priv->window, priv->icon);
 		empathy_conf_set_bool (empathy_conf_get (),
 				      EMPATHY_PREFS_UI_MAIN_WINDOW_HIDDEN, TRUE);
@@ -423,6 +420,16 @@ status_icon_toggle_visibility (EmpathyStatusIcon *icon)
 	}
 }
 
+static void
+status_icon_toggle_visibility (EmpathyStatusIcon *icon)
+{
+	EmpathyStatusIconPriv *priv = GET_PRIV (icon);
+	gboolean               visible;
+
+	visible = gtk_window_is_active (priv->window);
+	status_icon_set_visibility (icon, !visible);
+}
+
 static void
 status_icon_activate_cb (GtkStatusIcon     *status_icon,
 			 EmpathyStatusIcon *icon)
@@ -446,7 +453,7 @@ status_icon_delete_event_cb (GtkWidget         *widget,
 			     GdkEvent          *event,
 			     EmpathyStatusIcon *icon)
 {
-	status_icon_toggle_visibility (icon);
+	status_icon_set_visibility (icon, FALSE);
 
 	return TRUE;
 }
@@ -505,14 +512,11 @@ status_icon_create_menu (EmpathyStatusIcon *icon)
 
 	empathy_glade_connect (glade,
 			      icon,
+			      "tray_show_list", "toggled", status_icon_show_hide_window_cb,
 			      "tray_new_message", "activate", status_icon_new_message_cb,
 			      "tray_quit", "activate", status_icon_quit_cb,
 			      NULL);
 
-	g_signal_connect (priv->show_window_item, "toggled",
-			  G_CALLBACK (status_icon_show_hide_window_cb),
-			  icon);
-
 	g_object_unref (glade);
 }
 
@@ -538,7 +542,10 @@ static void
 status_icon_show_hide_window_cb (GtkWidget         *widget,
 				 EmpathyStatusIcon *icon)
 {
-	status_icon_toggle_visibility (icon);
+	gboolean visible;
+
+	visible = gtk_check_menu_item_get_active (GTK_CHECK_MENU_ITEM (widget));
+	status_icon_set_visibility (icon, visible);
 }
 
 static void
