@@ -28,8 +28,8 @@
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 
-#include <libgnome/gnome-program.h>
-#include <libgnomeui/gnome-ui-init.h>
+#include <libebook/e-book.h>
+#include <libgnomevfs/gnome-vfs.h>
 
 #include <libtelepathy/tp-conn.h>
 #include <libtelepathy/tp-chan.h>
@@ -37,17 +37,20 @@
 #include <libmissioncontrol/mc-account-monitor.h>
 #include <libmissioncontrol/mission-control.h>
 
-#include <libempathy/gossip-debug.h>
-#include <libempathy/gossip-utils.h>
-#include <libempathy/gossip-presence.h>
-#include <libempathy/gossip-contact.h>
+#include <libempathy/empathy-debug.h>
+#include <libempathy/empathy-utils.h>
+#include <libempathy/empathy-presence.h>
+#include <libempathy/empathy-contact.h>
 #include <libempathy/empathy-chandler.h>
 #include <libempathy/empathy-tp-chat.h>
+#include <libempathy/empathy-tp-chatroom.h>
 #include <libempathy/empathy-idle.h>
+#include <libempathy/empathy-conf.h>
+#include <libempathy-gtk/empathy-preferences.h>
 #include <libempathy-gtk/empathy-main-window.h>
 #include <libempathy-gtk/empathy-status-icon.h>
-#include <libempathy-gtk/gossip-private-chat.h>
-#include <libempathy-gtk/gossip-group-chat.h>
+#include <libempathy-gtk/empathy-private-chat.h>
+#include <libempathy-gtk/empathy-group-chat.h>
 
 #define DEBUG_DOMAIN "EmpathyMain"
 
@@ -58,7 +61,7 @@ static void
 service_ended_cb (MissionControl *mc,
 		  gpointer        user_data)
 {
-	gossip_debug (DEBUG_DOMAIN, "Mission Control stopped");
+	empathy_debug (DEBUG_DOMAIN, "Mission Control stopped");
 }
 
 static void
@@ -67,7 +70,7 @@ operation_error_cb (MissionControl *mc,
 		    guint           error_code,
 		    gpointer        user_data)
 {
-	gossip_debug (DEBUG_DOMAIN, "Error code %d during operation %d",
+	empathy_debug (DEBUG_DOMAIN, "Error code %d during operation %d",
 		      error_code,
 		      operation_id);
 }
@@ -92,7 +95,7 @@ account_enabled_cb (McAccountMonitor *monitor,
 		    gchar            *unique_name,
 		    EmpathyIdle      *idle)
 {
-	gossip_debug (DEBUG_DOMAIN, "Account enabled: %s", unique_name);
+	empathy_debug (DEBUG_DOMAIN, "Account enabled: %s", unique_name);
 	start_mission_control (idle);
 }
 
@@ -103,25 +106,29 @@ new_channel_cb (EmpathyChandler *chandler,
 		MissionControl  *mc)
 {
 	McAccount  *account;
-	GossipChat *chat;
+	EmpathyChat *chat;
 	gchar      *id;
 
 	account = mission_control_get_account_for_connection (mc, tp_conn, NULL);
-	id = gossip_get_channel_id (account, tp_chan);
-	chat = gossip_chat_window_find_chat (account, id);
+	id = empathy_inspect_channel (account, tp_chan);
+	chat = empathy_chat_window_find_chat (account, id);
 	g_free (id);
 
 	if (chat) {
 		/* The chat already exists */
-		if (!gossip_chat_is_connected (chat)) {
+		if (!empathy_chat_is_connected (chat)) {
 			EmpathyTpChat *tp_chat;
 
 			/* The chat died, give him the new text channel */
-			tp_chat = empathy_tp_chat_new (account, tp_chan);
-			gossip_chat_set_tp_chat (chat, tp_chat);
+			if (empathy_chat_is_group_chat (chat)) {
+				tp_chat = EMPATHY_TP_CHAT (empathy_tp_chatroom_new (account, tp_chan));
+			} else {
+				tp_chat = empathy_tp_chat_new (account, tp_chan);
+			}
+			empathy_chat_set_tp_chat (chat, tp_chat);
 			g_object_unref (tp_chat);
 		}
-		gossip_chat_present (chat);
+		empathy_chat_present (chat);
 
 		g_object_unref (account);
 		return;
@@ -129,19 +136,125 @@ new_channel_cb (EmpathyChandler *chandler,
 
 	if (tp_chan->handle_type == TP_HANDLE_TYPE_CONTACT) {
 		/* We have a new private chat channel */
-		chat = GOSSIP_CHAT (gossip_private_chat_new (account, tp_chan));
+		chat = EMPATHY_CHAT (empathy_private_chat_new (account, tp_chan));
 	}
 	else if (tp_chan->handle_type == TP_HANDLE_TYPE_ROOM) {
 		/* We have a new group chat channel */
-		chat = GOSSIP_CHAT (gossip_group_chat_new (account, tp_chan));
+		chat = EMPATHY_CHAT (empathy_group_chat_new (account, tp_chan));
 	}
 
-	gossip_chat_present (GOSSIP_CHAT (chat));
+	empathy_chat_present (EMPATHY_CHAT (chat));
 
 	g_object_unref (chat);
 	g_object_unref (account);
 }
 
+static void
+create_salut_account (void)
+{
+	McProfile  *profile;
+	McProtocol *protocol;
+	gboolean    salut_created;
+	McAccount  *account;
+	GList      *accounts;
+	EBook      *book;
+	EContact   *contact;
+	gchar      *nickname = NULL;
+	gchar      *published_name = NULL;
+	gchar      *first_name = NULL;
+	gchar      *last_name = NULL;
+	gchar      *email = NULL;
+	gchar      *jid = NULL;
+
+	/* Check if we already created a salut account */
+	if (!empathy_conf_get_bool (empathy_conf_get(),
+				    EMPATHY_PREFS_SALUT_ACCOUNT_CREATED,
+				    &salut_created)) {
+		return;
+	}
+	if (salut_created) {
+		return;
+	}
+
+	empathy_debug (DEBUG_DOMAIN, "Try to add a salut account...");
+
+	/* Check if the salut CM is installed */
+	profile = mc_profile_lookup ("salut");
+	protocol = mc_profile_get_protocol (profile);
+	if (!protocol) {
+		empathy_debug (DEBUG_DOMAIN, "Salut not installed");
+		g_object_unref (profile);
+		return;
+	}
+	g_object_unref (protocol);
+
+	/* Get self EContact from EDS */
+	if (!e_book_get_self (&contact, &book, NULL)) {
+		empathy_debug (DEBUG_DOMAIN, "Failed to get self econtact");
+		g_object_unref (profile);
+		return;
+	}
+
+	empathy_conf_set_bool (empathy_conf_get (),
+			       EMPATHY_PREFS_SALUT_ACCOUNT_CREATED,
+			       TRUE);
+
+	/* Check if there is already a salut account */
+	accounts = mc_accounts_list_by_profile (profile);
+	if (accounts) {
+		empathy_debug (DEBUG_DOMAIN, "There is already a salut account");
+		mc_accounts_list_free (accounts);
+		g_object_unref (profile);
+		return;
+	}
+
+	account = mc_account_create (profile);
+	mc_account_set_display_name (account, _("People nearby"));
+	
+	nickname = e_contact_get (contact, E_CONTACT_NICKNAME);
+	published_name = e_contact_get (contact, E_CONTACT_FULL_NAME);
+	first_name = e_contact_get (contact, E_CONTACT_GIVEN_NAME);
+	last_name = e_contact_get (contact, E_CONTACT_FAMILY_NAME);
+	email = e_contact_get (contact, E_CONTACT_EMAIL_1);
+	jid = e_contact_get (contact, E_CONTACT_IM_JABBER_HOME_1);
+	
+	if (G_STR_EMPTY (nickname) || !empathy_strdiff (nickname, "nickname")) {
+		g_free (nickname);
+		nickname = g_strdup (g_get_user_name ());
+	}
+	if (G_STR_EMPTY (published_name)) {
+		g_free (published_name);
+		published_name = g_strdup (g_get_real_name ());
+	}
+
+	empathy_debug (DEBUG_DOMAIN, "Salut account created:\n"
+				     "  nickname=%s\n"
+				     "  published-name=%s\n"
+				     "  first-name=%s\n"
+				     "  last-name=%s\n"
+				     "  email=%s\n"
+				     "  jid=%s\n",
+		       nickname, published_name, first_name, last_name, email, jid);
+
+	mc_account_set_param_string (account, "nickname", nickname ? nickname : "");
+	mc_account_set_param_string (account, "published-name", published_name ? published_name : "");
+	mc_account_set_param_string (account, "first-name", first_name ? first_name : "");
+	mc_account_set_param_string (account, "last-name", last_name ? last_name : "");
+	mc_account_set_param_string (account, "email", email ? email : "");
+	mc_account_set_param_string (account, "jid", jid ? jid : "");
+
+	g_free (nickname);
+	g_free (published_name);
+	g_free (first_name);
+	g_free (last_name);
+	g_free (email);
+	g_free (jid);
+	g_object_unref (account);
+	g_object_unref (profile);
+	g_object_unref (contact);
+	g_object_unref (book);
+}
+
 int
 main (int argc, char *argv[])
 {
@@ -151,7 +264,6 @@ main (int argc, char *argv[])
 	McAccountMonitor  *monitor;
 	EmpathyIdle       *idle;
 	EmpathyChandler   *chandler;
-	GnomeProgram      *program;
 	gboolean           no_connect = FALSE;
 	GOptionContext    *context;
 	GOptionEntry       options[] = {
@@ -162,6 +274,8 @@ main (int argc, char *argv[])
 		{ NULL }
 	};
 
+	empathy_debug_set_log_file_from_env ();
+
 	bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
@@ -171,22 +285,15 @@ main (int argc, char *argv[])
 
 	g_set_application_name (PACKAGE_NAME);
 
-	program = gnome_program_init ("empathy",
-				      PACKAGE_VERSION,
-				      LIBGNOMEUI_MODULE,
-				      argc, argv,
-				      GNOME_PROGRAM_STANDARD_PROPERTIES,
-				      "goption-context", context,
-				      GNOME_PARAM_HUMAN_READABLE_NAME, PACKAGE_NAME,
-				      NULL);
-
+	gtk_init (&argc, &argv);
 	gtk_window_set_default_icon_name ("empathy");
 	gtk_icon_theme_append_search_path (gtk_icon_theme_get_default (),
-					   DATADIR G_DIR_SEPARATOR_S "empathy");
+					   PKGDATADIR G_DIR_SEPARATOR_S "icons");
+	gnome_vfs_init ();
 
 	/* Setting up MC */
 	monitor = mc_account_monitor_new ();
-	mc = gossip_mission_control_new ();
+	mc = empathy_mission_control_new ();
 	idle = empathy_idle_new ();
 	g_signal_connect (monitor, "account-enabled",
 			  G_CALLBACK (account_enabled_cb),
@@ -201,6 +308,8 @@ main (int argc, char *argv[])
 	if (!no_connect) {
 		start_mission_control (idle);
 	}
+	
+	create_salut_account ();
 
 	/* Setting up UI */
 	window = empathy_main_window_show ();
@@ -221,7 +330,6 @@ main (int argc, char *argv[])
 	g_object_unref (mc);
 	g_object_unref (idle);
 	g_object_unref (icon);
-	g_object_unref (program);
 
 	return EXIT_SUCCESS;
 }
